@@ -41,6 +41,9 @@
 #include "xrdp_sockets.h"
 #include "audin.h"
 
+#include "scp.h"
+#include "scp_sync.h"
+
 #include "ms-rdpbcgr.h"
 
 #define MAX_PATH 260
@@ -1592,20 +1595,32 @@ get_log_path(char *path, int bytes)
     int rv;
 
     rv = 1;
-    log_path = g_getenv("CHANSRV_LOG_PATH");
-    if (log_path == 0)
+    if (g_cfg->log_file_path != NULL && g_cfg->log_file_path[0] != '\0')
     {
-        log_path = g_getenv("XDG_DATA_HOME");
-        if (log_path != 0)
+        char uidstr[64];
+        char username[64];
+        const struct info_string_tag map[] =
         {
-            g_snprintf(path, bytes, "%s%s", log_path, "/xrdp");
-            if (g_directory_exist(path) || (g_mkdir(path) == 0))
-            {
-                rv = 0;
-            }
+            {'u', uidstr},
+            {'U', username},
+            INFO_STRING_END_OF_LIST
+        };
+
+        int uid = g_getuid();
+        g_snprintf(uidstr, sizeof(uidstr), "%d", uid);
+        if (g_getlogin(username, sizeof(username)) != 0)
+        {
+            /* Fall back to UID */
+            g_strncpy(username, uidstr, sizeof(username) - 1);
+        }
+
+        (void)g_format_info_string(path, bytes, g_cfg->log_file_path, map);
+        if (g_directory_exist(path) || (g_mkdir(path) == 0))
+        {
+            rv = 0;
         }
     }
-    else
+    else if ((log_path = g_getenv("CHANSRV_LOG_PATH")) != 0)
     {
         g_snprintf(path, bytes, "%s", log_path);
         if (g_directory_exist(path) || (g_mkdir(path) == 0))
@@ -1613,6 +1628,16 @@ get_log_path(char *path, int bytes)
             rv = 0;
         }
     }
+    else if ((log_path = g_getenv("XDG_DATA_HOME")) != 0)
+    {
+        g_snprintf(path, bytes, "%s%s", log_path, "/xrdp");
+        if (g_directory_exist(path) || (g_mkdir(path) == 0))
+        {
+            rv = 0;
+        }
+    }
+
+    // Always fall back to the home directory
     if (rv != 0)
     {
         log_path = g_getenv("HOME");
@@ -1664,6 +1689,54 @@ run_exec(void)
     return 0;
 }
 
+/*****************************************************************************/
+/**
+ * Make sure XRDP_SOCKET_PATH exists
+ *
+ * We can't do anything without XRDP_SOCKET_PATH existing.
+ *
+ * Normally this is done by sesman before chansrv starts. If we're running
+ * standalone however (i.e. with x11vnc) this won't be done. We don't have the
+ * privilege to create the directory, so we have to ask sesman to do it
+ * for us.
+ */
+static int
+chansrv_create_xrdp_socket_path(void)
+{
+    char xrdp_socket_path[XRDP_SOCKETS_MAXPATH];
+    int rv = 1;
+
+    /* Use our UID to qualify XRDP_SOCKET_PATH */
+    g_snprintf(xrdp_socket_path, sizeof(xrdp_socket_path),
+               XRDP_SOCKET_PATH, g_getuid());
+
+    if (g_directory_exist(xrdp_socket_path))
+    {
+        rv = 0;
+    }
+    else
+    {
+        LOG(LOG_LEVEL_INFO, "%s doesn't exist - asking sesman to create it",
+            xrdp_socket_path);
+
+        struct trans *t = NULL;
+
+        if (!(t = scp_connect(g_cfg->listen_port, "xrdp-chansrv", g_is_term)))
+        {
+            LOG(LOG_LEVEL_ERROR, "Can't connect to sesman");
+        }
+        else if (scp_sync_uds_login_request(t) == 0 &&
+                 scp_sync_create_sockdir_request(t) == 0)
+        {
+            rv = 0;
+            (void)scp_send_close_connection_request(t);
+        }
+        trans_delete(t);
+    }
+
+    return rv;
+}
+
 /*****************************************************************************/
 int
 main(int argc, char **argv)
@@ -1680,14 +1753,6 @@ main(int argc, char **argv)
     g_init("xrdp-chansrv"); /* os_calls */
     g_memset(g_drdynvcs, 0, sizeof(g_drdynvcs));
 
-    log_path[255] = 0;
-    if (get_log_path(log_path, 255) != 0)
-    {
-        g_writeln("error reading CHANSRV_LOG_PATH and HOME environment variable");
-        main_cleanup();
-        return 1;
-    }
-
     display_text = g_getenv("DISPLAY");
     if (display_text == NULL)
     {
@@ -1714,6 +1779,13 @@ main(int argc, char **argv)
     }
     config_dump(g_cfg);
 
+    if (get_log_path(log_path, sizeof(log_path)) != 0)
+    {
+        g_writeln("error reading CHANSRV_LOG_PATH and HOME environment variable");
+        main_cleanup();
+        return 1;
+    }
+
     pid = g_getpid();
 
     /* starting logging subsystem */
@@ -1756,6 +1828,13 @@ main(int argc, char **argv)
     }
 
     LOG_DEVEL(LOG_LEVEL_INFO, "main: app started pid %d(0x%8.8x)", pid, pid);
+
+    if (chansrv_create_xrdp_socket_path() != 0)
+    {
+        main_cleanup();
+        return 1;
+    }
+
     /*  set up signal handler  */
     g_signal_terminate(term_signal_handler); /* SIGTERM */
     g_signal_user_interrupt(term_signal_handler); /* SIGINT */
