@@ -40,11 +40,13 @@
 #define DEFAULT_RESTRICT_INBOUND_CLIPBOARD  0
 #define DEFAULT_ENABLE_FUSE_MOUNT           1
 #define DEFAULT_FUSE_MOUNT_NAME             "xrdp-client"
+#define DEFAULT_FUSE_DIRECT_IO              0
 #define DEFAULT_FILE_UMASK                  077
 #define DEFAULT_USE_NAUTILUS3_FLIST_FORMAT  0
 #define DEFAULT_NUM_SILENT_FRAMES_AAC       4
 #define DEFAULT_NUM_SILENT_FRAMES_MP3       2
 #define DEFAULT_MSEC_DO_NOT_SEND            1000
+#define DEFAULT_LOG_FILE_PATH               ""
 /**
  * Type used for passing a logging function about
  */
@@ -72,6 +74,61 @@ log_to_stdout(const enum logLevels lvl, const char *msg, ...)
     return LOG_STARTUP_OK;
 }
 
+/***************************************************************************//**
+ * Reads the config values we need from the [Globals] section
+ *
+ * @param logmsg Function to use to log messages
+ * @param names List of definitions in the section
+ * @params values List of corresponding values for the names
+ * @params cfg Pointer to structure we're filling in
+ *
+ * @return 0 for success
+ */
+static int
+read_config_globals(log_func_t logmsg,
+                    struct list *names, struct list *values,
+                    struct config_chansrv *cfg)
+{
+    int error = 0;
+    int index;
+
+    for (index = 0; index < names->count; ++index)
+    {
+        const char *name = (const char *)list_get_item(names, index);
+        const char *value = (const char *)list_get_item(values, index);
+
+        char unrecognised[256];
+        if (g_strcasecmp(name, "ListenPort") == 0)
+        {
+            char *listen_port = strdup(value);
+            if (listen_port == NULL)
+            {
+                LOG(LOG_LEVEL_WARNING,
+                    "Can't allocate config memory for ListenPort");
+            }
+            else
+            {
+                g_free(cfg->listen_port);
+                cfg->listen_port = listen_port;
+            }
+        }
+        if (g_strcasecmp(name, "RestrictInboundClipboard") == 0)
+        {
+            cfg->restrict_inbound_clipboard =
+                sesman_clip_restrict_string_to_bitmask(
+                    value, unrecognised, sizeof(unrecognised));
+            if (unrecognised[0] != '\0')
+            {
+                LOG(LOG_LEVEL_WARNING,
+                    "Unrecognised tokens parsing 'RestrictInboundClipboard' %s",
+                    unrecognised);
+            }
+        }
+    }
+
+    return error;
+}
+
 /***************************************************************************//**
  * Reads the config values we need from the [Security] section
  *
@@ -163,6 +220,10 @@ read_config_chansrv(log_func_t logmsg,
                 break;
             }
         }
+        else if (g_strcasecmp(name, "FuseDirectIO") == 0)
+        {
+            cfg->fuse_direct_io = g_text2bool(value);
+        }
         else if (g_strcasecmp(name, "FileUmask") == 0)
         {
             cfg->file_umask = strtol(value, NULL, 0);
@@ -188,6 +249,45 @@ read_config_chansrv(log_func_t logmsg,
     return error;
 }
 
+/***************************************************************************//**
+ * Reads the config values we need from the [ChansrvLogging] section
+ *
+ * @param logmsg Function to use to log messages
+ * @param names List of definitions in the section
+ * @params values List of corresponding values for the names
+ * @params cfg Pointer to structure we're filling in
+ *
+ * @return 0 for success
+ */
+static int
+read_config_chansrv_logging(log_func_t logmsg,
+                            struct list *names, struct list *values,
+                            struct config_chansrv *cfg)
+{
+    int error = 0;
+    int index;
+
+    for (index = 0; index < names->count; ++index)
+    {
+        const char *name = (const char *)list_get_item(names, index);
+        const char *value = (const char *)list_get_item(values, index);
+
+        if (g_strcasecmp(name, "LogFilePath") == 0)
+        {
+            g_free(cfg->log_file_path);
+            cfg->log_file_path = g_strdup(value);
+            if (cfg->log_file_path == NULL)
+            {
+                logmsg(LOG_LEVEL_ERROR, "Can't alloc LogFilePath");
+                error = 1;
+                break;
+            }
+        }
+    }
+
+    return error;
+}
+
 /***************************************************************************//**
  * @brief returns a config block with default values
  *
@@ -199,24 +299,29 @@ new_config(void)
     /* Do all the allocations at the beginning, then check them together */
     struct config_chansrv *cfg = g_new0(struct config_chansrv, 1);
     char *fuse_mount_name = g_strdup(DEFAULT_FUSE_MOUNT_NAME);
-    if (cfg == NULL || fuse_mount_name == NULL)
+    char *log_file_path = g_strdup(DEFAULT_LOG_FILE_PATH);
+    if (cfg == NULL || fuse_mount_name == NULL || log_file_path == NULL)
     {
         /* At least one memory allocation failed */
+        g_free(log_file_path);
         g_free(fuse_mount_name);
         g_free(cfg);
         cfg = NULL;
     }
     else
     {
+        cfg->listen_port = NULL;
         cfg->enable_fuse_mount = DEFAULT_ENABLE_FUSE_MOUNT;
         cfg->restrict_outbound_clipboard = DEFAULT_RESTRICT_OUTBOUND_CLIPBOARD;
         cfg->restrict_inbound_clipboard = DEFAULT_RESTRICT_INBOUND_CLIPBOARD;
         cfg->fuse_mount_name = fuse_mount_name;
+        cfg->fuse_direct_io = DEFAULT_FUSE_DIRECT_IO;
         cfg->file_umask = DEFAULT_FILE_UMASK;
         cfg->use_nautilus3_flist_format = DEFAULT_USE_NAUTILUS3_FLIST_FORMAT;
         cfg->num_silent_frames_aac = DEFAULT_NUM_SILENT_FRAMES_AAC;
         cfg->num_silent_frames_mp3 = DEFAULT_NUM_SILENT_FRAMES_MP3;
         cfg->msec_do_not_send = DEFAULT_MSEC_DO_NOT_SEND;
+        cfg->log_file_path = log_file_path;
     }
 
     return cfg;
@@ -252,6 +357,11 @@ config_read(int use_logger, const char *sesman_ini)
             names->auto_free = 1;
             values->auto_free = 1;
 
+            if (!error && file_read_section(fd, "Globals", names, values) == 0)
+            {
+                error = read_config_globals(logmsg, names, values, cfg);
+            }
+
             if (!error && file_read_section(fd, "Security", names, values) == 0)
             {
                 error = read_config_security(logmsg, names, values, cfg);
@@ -262,6 +372,12 @@ config_read(int use_logger, const char *sesman_ini)
                 error = read_config_chansrv(logmsg, names, values, cfg);
             }
 
+            if (!error &&
+                    file_read_section(fd, "ChansrvLogging", names, values) == 0)
+            {
+                error = read_config_chansrv_logging(logmsg, names, values, cfg);
+            }
+
             list_delete(names);
             list_delete(values);
         }
@@ -282,9 +398,12 @@ config_read(int use_logger, const char *sesman_ini)
 void
 config_dump(struct config_chansrv *config)
 {
-    g_writeln("Global configuration:");
-
     char buf[256];
+
+    g_writeln("Global configuration:");
+    g_writeln("    xrdp-sesman ListenPort:    %s",
+              (config->listen_port) ? config->listen_port : "<default>");
+
     g_writeln("\nSecurity configuration:");
     sesman_clip_restrict_mask_to_string(
         config->restrict_outbound_clipboard,
@@ -300,9 +419,13 @@ config_dump(struct config_chansrv *config)
     g_writeln("    EnableFuseMount            %s",
               g_bool2text(config->enable_fuse_mount));
     g_writeln("    FuseMountName:             %s", config->fuse_mount_name);
+    g_writeln("    FuseDirectIO:              %s",
+              g_bool2text(config->fuse_direct_io));
     g_writeln("    FileMask:                  0%o", config->file_umask);
     g_writeln("    Nautilus 3 Flist Format:   %s",
               g_bool2text(config->use_nautilus3_flist_format));
+    g_writeln("    LogFilePath            :   %s",
+              (config->log_file_path[0]) ? config->log_file_path : "<default>");
 }
 
 /******************************************************************************/
@@ -311,7 +434,9 @@ config_free(struct config_chansrv *cc)
 {
     if (cc != NULL)
     {
+        g_free(cc->listen_port);
         g_free(cc->fuse_mount_name);
+        g_free(cc->log_file_path);
         g_free(cc);
     }
 }
