@@ -170,6 +170,22 @@ lib_mod_connect(struct mod *mod)
         return 1;
     }
 
+
+    // This is a good place to finalise any parameters that need to
+    // be set.
+    if (mod->client_info.h264_frame_interval <= 0)
+    {
+        mod->client_info.h264_frame_interval = DEFAULT_H264_FRAME_INTERVAL;
+    }
+    if (mod->client_info.rfx_frame_interval <= 0)
+    {
+        mod->client_info.rfx_frame_interval = DEFAULT_RFX_FRAME_INTERVAL;
+    }
+    if (mod->client_info.normal_frame_interval <= 0)
+    {
+        mod->client_info.normal_frame_interval = DEFAULT_NORMAL_FRAME_INTERVAL;
+    }
+
     make_stream(s);
     g_sprintf(con_port, "%s", mod->port);
 
@@ -1855,6 +1871,18 @@ lib_mod_set_param(struct mod *mod, const char *name, const char *value)
     {
         g_strncpy(mod->port, value, 255);
     }
+    else if (g_strcasecmp(name, "h264_frame_interval") == 0)
+    {
+        mod->client_info.h264_frame_interval = g_atoi(value);
+    }
+    else if (g_strcasecmp(name, "rfx_frame_interval") == 0)
+    {
+        mod->client_info.rfx_frame_interval = g_atoi(value);
+    }
+    else if (g_strcasecmp(name, "normal_frame_interval") == 0)
+    {
+        mod->client_info.normal_frame_interval = g_atoi(value);
+    }
     else if (g_strcasecmp(name, "client_info") == 0)
     {
         g_memcpy(&(mod->client_info), value, sizeof(mod->client_info));
